// Generated by CoffeeScript 1.9.3
var args, bits, cmd, compile, compiler, compilers, conf, confs, credentials, cwd, defaultExclude, err, error, exec, ext, fs, i, j, len, len1, log, mod, opt, opts, os, p, ref, ref1, ref2, ref3, requireConfig, server, usage, version, vigil, vigilOpts;

exec = require('executive');

fs = require('fs');

os = require('os');

vigil = require('vigil');

compilers = require('./compilers');

log = require('./log');

server = require('./server');

defaultExclude = require('./utils').defaultExclude;

error = function(message) {
  log.error(message);
  return process.exit(1);
};

version = function() {
  console.log((require('../package.json')).version);
  return process.exit(0);
};

usage = function() {
  console.log("bebop [options] [file]\n\nOptions:\n  --compile, -c   Compile files and exit\n  --compilers,    Specify compiler to use for a given extension\n  --config,       Specify bebop.coffee to use\n  --exclude, -x   Exclude files for watching, compiling\n  --force-reload  Force reload when file is compiled\n  --host, -h      Hostname to bind to\n  --include, -i   Include files for watching, compiling\n  --no-compile    Do not compile files automatically\n  --no-server     Do not run static file server\n  --no-watch      Do not watch files for changes\n  --open, -o      Open browser automatically\n  --port, -p      Port to listen on\n  --pre           Command to execute first\n  --secure, -s    Require authentication\n  --static-dir    Directory used as root for static file server\n  --work-dir      Directory used as root for compiling, watching\n\n  --help          Display this message\n  --version, -v   Display version");
  return process.exit(0);
};

try {
  require('coffee-script/register');
} catch (_error) {
  err = _error;
}

cwd = process.cwd();

confs = [process.env.HOME + '/.bebop', cwd + '/.bebop', cwd + '/bebop'];

opts = {
  compile: true,
  compileOnly: false,
  compilers: {},
  defaultExclude: true,
  exclude: [],
  forceReload: false,
  host: 'localhost',
  include: [],
  index: '',
  port: null,
  pre: function(done) {
    return done();
  },
  runServer: true,
  staticDir: cwd,
  workDir: cwd,
  watch: true
};

requireConfig = function(path) {
  var conf, k, ref, results, v;
  try {
    conf = require.resolve(path);
  } catch (_error) {
    err = _error;
    return;
  }
  if (fs.existsSync(conf)) {
    ref = require(conf);
    results = [];
    for (k in ref) {
      v = ref[k];
      results.push(opts[k] = v);
    }
    return results;
  }
};

for (i = 0, len = confs.length; i < len; i++) {
  conf = confs[i];
  requireConfig(conf);
}

args = process.argv.slice(2);

while (opt = args.shift()) {
  switch (opt) {
    case '--help':
      usage();
      break;
    case '--version':
    case '-v':
      version();
      break;
    case '--config':
      requireConfig(args.shift());
      break;
    case '--open':
    case '-o':
      opts.open = true;
      break;
    case '--no-server':
      opts.runServer = false;
      break;
    case '--no-watch':
      opts.watch = false;
      break;
    case '--no-compile':
      opts.compile = false;
      break;
    case '--compile':
    case '-c':
      opts.compileOnly = true;
      break;
    case '--include':
    case '-i':
      opts.include.push(new RegExp(args.shift()));
      break;
    case '--exclude':
    case '-x':
      opts.exclude.push(new RegExp(args.shift()));
      break;
    case '--no-default-exclude':
      opts.defaultExclude = false;
      break;
    case '--force-reload':
      opts.forceReload = true;
      break;
    case '--host':
    case '-h':
      opts.host = args.shift();
      break;
    case '--pre':
      cmd = args.shift();
      opts.pre = function(done) {
        return exec(cmd, done);
      };
      break;
    case '--port':
    case '-p':
      p = args.shift();
      if (!p) {
        error('missing port');
      }
      opts.port = parseInt(p, 10);
      break;
    case '--secure':
    case '-s':
      credentials = args.shift();
      if (credentials) {
        ref = credentials.split(':'), opts.user = ref[0], opts.pass = ref[1];
      } else {
        opts.user = 'bebop';
        opts.pass = 'beepboop';
      }
      break;
    case '--static-dir':
      opts.staticDir = args.shift();
      break;
    case '--work-dir':
      opts.workDir = args.shift();
      break;
    case '--compilers':
    case '-c':
      ref1 = args.shift().split(',');
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        compiler = ref1[j];
        ref2 = compiler.split(':'), ext = ref2[0], mod = ref2[1];
        opts.compilers[ext] = mod;
      }
      break;
    default:
      if (opt.charAt(0) === '-') {
        error('Unrecognized option');
      } else {
        opts.index = opt;
      }
  }
}

if (opts.defaultExclude) {
  opts.exclude = [vigil.utils.excludeRe, defaultExclude].concat(opts.exclude);
}

if (opts.include.length === 0) {
  opts.include = null;
}

vigilOpts = {
  exclude: opts.exclude,
  include: opts.include,
  patch: false
};

ref3 = opts.compilers;
for (ext in ref3) {
  compiler = ref3[ext];
  if (typeof compiler === 'string') {
    try {
      bits = compiler.split('.');
      compiler = require(bits.shift());
      while (bits.length) {
        compiler = compiler[bits.shift()];
      }
      compilers[ext] = compiler;
    } catch (_error) {
      err = _error;
      console.log(err);
    }
  } else {
    compilers[ext] = compiler;
  }
}

compile = function(filename, cb) {
  if (cb == null) {
    cb = function() {};
  }
  return compilers.compile(filename, function(err, compiled) {
    if (filename.indexOf(opts.workDir === 0)) {
      filename = (filename.replace(opts.workDir, '')).replace(/^\//, '');
    }
    if (err != null) {
      log.error("failed to compile " + filename);
      console.error(err.stderr != null ? err.stderr : err.stack);
      return;
    }
    if (compiled) {
      log.compiled(filename);
    }
    return cb(null, compiled);
  });
};

opts.pre(function(err) {
  var app, websocket;
  if (err != null) {
    return;
  }
  if (opts.compile) {
    vigil.walk(opts.workDir, vigilOpts, function(filename) {
      if (opts.compile) {
        return compile(filename);
      }
    });
  }
  if (opts.compileOnly) {
    return;
  }
  if (opts.runServer) {
    app = server.createServer(opts);
    websocket = (require('./websocket'))({
      server: app
    });
  } else {
    app = {
      run: function() {}
    };
    websocket = {
      modified: function() {}
    };
  }
  if (opts.watch) {
    vigil.watch(opts.workDir, vigilOpts, function(filename, stat, isModule) {
      if (!opts.compile) {
        log.modified(filename);
        return websocket.modified(filename);
      }
      return compile(filename, function(err, compiled) {
        if (!compiled) {
          log.modified(filename);
          return websocket.modified(filename);
        } else {
          if (opts.forceReload) {
            return websocket.modified(filename);
          }
        }
      });
    });
  }
  return app.run(function() {
    if (opts.open || opts.index !== '') {
      if (opts.index === 'index.html') {
        opts.index = '';
      }
      switch (os.platform()) {
        case 'darwin':
          return exec("open http://" + opts.host + ":" + opts.port + "/" + opts.index);
        case 'linux':
          return exec("xdg-open http://" + opts.host + ":" + opts.port + "/" + opts.index);
      }
    }
  });
});
