// Generated by CoffeeScript 1.9.3
var Bebop, EventEmitter, WebSocket, dir, dump, findNode, log, ref, reloadNode, root, stacktrace,
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

EventEmitter = require('./event-emitter');

WebSocket = require('./websocket');

dir = require('./dir');

dump = require('./dump');

log = require('./log');

stacktrace = require('./stacktrace');

ref = require('./node'), reloadNode = ref.reloadNode, findNode = ref.findNode;

root = require('./utils').root;

Bebop = (function(superClass) {
  extend(Bebop, superClass);

  function Bebop(opts) {
    var hostname, port, protocol, ref1, ref2, ref3, ref4, ref5, ref6;
    if (opts == null) {
      opts = {};
    }
    Bebop.__super__.constructor.apply(this, arguments);
    protocol = (ref1 = opts.protocol) != null ? ref1 : location.protocol === 'http:' ? 'ws://' : 'wss://';
    hostname = (ref2 = opts.hostname) != null ? ref2 : location.hostname;
    port = (ref3 = opts.port) != null ? ref3 : location.port;
    this.address = (ref4 = opts.address) != null ? ref4 : protocol + hostname + ':' + port + '/_bebop';
    this.timeout = (ref5 = opts.timeout) != null ? ref5 : 1000;
    this.debug = (ref6 = opts.debug) != null ? ref6 : false;
    this.tries = 0;
    this.limit = -1;
    this._failed = false;
    this._retries = [];
    this._once = {
      error: false,
      closed: false
    };
    this.init(opts);
  }

  Bebop.prototype.defaultHandlers = {
    connected: function() {
      log.info('connected');
      this._once = {
        error: false,
        closed: false
      };
      return this.sendConnected();
    },
    reconnecting: function() {
      return log.info('reconnecting');
    },
    closed: function() {
      if (!this._once.closed) {
        log.info('closed');
      }
      return this._once.closed = true;
    },
    error: function() {
      if (!this._once.error) {
        log.error('error');
      }
      return this._once.error = true;
    },
    message: function(message) {
      log.debug('message');
      switch (message.type) {
        case 'complete':
          return this.sendComplete(message.name);
        case 'eval':
          return this.sendEval(message.code);
        case 'modified':
          return this.modified(message.filename);
        case 'reload':
          return this.reload();
        default:
          return log.warn("Unknown message type '" + message.type + "'", message);
      }
    }
  };

  Bebop.prototype.init = function(opts) {
    var handlers, k, ref1, v;
    if (opts == null) {
      opts = {};
    }
    handlers = {
      connected: opts.onconnected,
      closed: opts.onclosed,
      error: opts.onerror,
      message: opts.onmessage,
      reconnecting: opts.reconnecting
    };
    ref1 = this.defaultHandlers;
    for (k in ref1) {
      v = ref1[k];
      if (handlers[k] == null) {
        handlers[k] = v;
      }
    }
    this.on('connect', (function(_this) {
      return function() {
        if ((_this.limit != null) && _this.limit >= 0) {
          if (_this.tries > _this.limit) {
            log.error('Bebop.connect', 'connection-failed: giving up!');
            return _this._failed = true;
          } else {
            return _this.tries = _this.tries + 1;
          }
        }
      };
    })(this));
    this.on('connected', (function(_this) {
      return function() {
        _this.tries = 0;
        return handlers.connected.apply(_this, arguments);
      };
    })(this));
    this.on('closed', (function(_this) {
      return function() {
        return handlers.closed.apply(_this, arguments);
      };
    })(this));
    this.on('error', (function(_this) {
      return function() {
        return handlers.error.apply(_this, arguments);
      };
    })(this));
    this.on('message', (function(_this) {
      return function() {
        return handlers.message.apply(_this, arguments);
      };
    })(this));
    return this.on('reconnecting', (function(_this) {
      return function() {
        return handlers.reconnecting.apply(_this, arguments);
      };
    })(this));
  };

  Bebop.prototype.connect = function() {
    var err;
    if (this._failed) {
      return;
    }
    this.emit('connecting');
    try {
      this.ws = new WebSocket(this.address);
    } catch (_error) {
      err = _error;
      log.warn('Failed to create WebSocket', err);
      return this.reconnect();
    }
    this.ws.onopen = (function(_this) {
      return function() {
        var args;
        _this.stopRetrying();
        args = Array.prototype.slice.call(arguments);
        args.unshift('connected');
        return _this.emit.apply(_this, args);
      };
    })(this);
    this.ws.onclose = (function(_this) {
      return function() {
        var args;
        args = Array.prototype.slice.call(arguments);
        args.unshift('closed');
        return _this.emit.apply(_this, args);
      };
    })(this);
    this.ws.onerror = (function(_this) {
      return function() {
        var args;
        args = Array.prototype.slice.call(arguments);
        args.unshift('error');
        return _this.emit.apply(_this, args);
      };
    })(this);
    this.ws.onmessage = (function(_this) {
      return function(e) {
        return _this.emit('message', JSON.parse(e.data));
      };
    })(this);
    if (root.isBrowser) {
      return root.addEventListener('beforeunload', (function(_this) {
        return function() {
          return _this.ws.close();
        };
      })(this));
    }
  };

  Bebop.prototype.reconnect = function() {
    if (this._failed) {
      return;
    }
    this.emit('reconnecting');
    return this._retries.push(root.setTimeout((function(_this) {
      return function() {
        return _this.connect();
      };
    })(this), this.timeout));
  };

  Bebop.prototype.stopRetrying = function() {
    var i, len, ref1, t;
    this.tries = 0;
    ref1 = this._retries;
    for (i = 0, len = ref1.length; i < len; i++) {
      t = ref1[i];
      clearTimeout(t);
    }
    return this._retries = [];
  };

  Bebop.prototype.close = function() {
    return this.ws.close();
  };

  Bebop.prototype.send = function(msg) {
    this.emit('send', msg);
    return this.ws.send(JSON.stringify(msg));
  };

  Bebop.prototype.complete = function(code) {
    var e;
    this.emit('complete', code);
    try {
      return dir(eval_.call(root, code));
    } catch (_error) {
      e = _error;
      return [];
    }
  };

  Bebop.prototype["eval"] = function(code) {
    var e, error, res;
    this.emit('eval', code);
    try {
      return res = eval_.call(root, msg);
    } catch (_error) {
      e = _error;
      return error = {
        error: e.message,
        stack: stacktrace(e)
      };
    }
  };

  Bebop.prototype.modified = function(filename) {
    var node;
    this.emit('modified', filename);
    if (isBrowser) {
      node = findNode(filename);
      if (node && node._resource.tag.name !== 'script') {
        return reloadNode(node);
      } else {
        return this.reload(true);
      }
    }
  };

  Bebop.prototype.reload = function() {
    this.emit('reload');
    this.close();
    return location.reload();
  };

  Bebop.prototype.sendConnected = function() {
    return this.send({
      type: 'connected',
      identifier: this.ws.identifier
    });
  };

  Bebop.prototype.sendComplete = function(code) {
    return this.send({
      type: 'complete',
      result: this.complete(code)
    });
  };

  Bebop.prototype.sendEval = function(code) {
    return this.send({
      type: 'eval',
      result: this["eval"](code)
    });
  };

  return Bebop;

})(EventEmitter);

module.exports = Bebop;
